/* DCJavaClient - a DirectConnect client written in Java
 * Copyright (C) 2001 Stefan Grling, stefan@gorling.se
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

import java.io.*;
import java.net.*;
import DCProtocolHandler;
import UserInfo;

/***
    This class handles connections.    

 ***/

class DCConnector {



    String host=null;
    int port=-1;
    boolean isConnected=false;
    boolean isListener=false;
    Socket connection=null;
    ServerSocket server=null;
    UserInfo myself=null;
    DCProtocolHandler protocolHandler=null;


    public DCConnector (String host, int port,UserInfo myself) {
	this.host=host;
	this.port=port;
	this.myself=myself;
    }


    public boolean isConnected () {
	return isConnected;
    }

    public boolean openConnection() {
	if (!isConnected) {
	    try {
		if (isListener) {
		    server=new ServerSocket(port);
		    connection=server.accept();
		    System.out.println("Listening on:"+port);
		}else{
		    System.out.println("Connecting to: "+host+":"+port);
		    connection=new Socket(host,port);		
		}

		if (connection==null) {
		    System.err.println("Warning - Socket is null!");
		}

		protocolHandler=new DCProtocolHandler(connection,myself);

		isConnected=true;

	    }catch(Exception e) {
		// Here we really should handle exceptions :)
		System.out.println("PCaught exception: "+e);
		return false;
	    }
 	}
      return true;
    }

    public boolean closeConnection() {
	if (isConnected) {
	    try{
		connection.close();
	    }catch(Exception e){
		return false;
	    }
	}
	    return true;
    }

    public boolean isLoggedIn() {
	return protocolHandler.isLoggedIn();
    }
    
    public DCProtocolHandler getProtocolHandler() {
	return protocolHandler;
    }

}







