/* DCJavaClient - a DirectConnect client written in Java
 * Copyright (C) 2001 Stefan Grling, stefan@gorling.se
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

import java.io.*;
import java.lang.*;


class DCEncryptionHandler {

/***
    @Description   This clas handles Encryption, i.e. we load the padding and replacement values and calculates responses on demand.
    @Version 0.20

***/

    char paddingValues[];
    char replacementValues[][];

    /***
	This class calculates the challenge-response process. 
	It should be made static when we've gotten the full table of keys.

	Eric Prevoteau <www@ac2i.tzo.com> has broken the algorithm in an nicer way, without the need for padding and replacement-values. We should implement this as time permits.
     ***/
    public DCEncryptionHandler() {
	loadValues();
    }
    
    private void loadValues() {

	// Clear the tables.
	paddingValues=new char[256];
	replacementValues=new char[256][256];
	for (int i=0;i<256;i++) {
	    for (int j=0;j<256;j++) replacementValues[i][j]=0;
	    paddingValues[i]=0;
	}

	try {
	    
	BufferedReader in = new BufferedReader(new FileReader("paddingCode.txt"));
	String indata=null;


	while(in.ready()) {
	    indata=in.readLine();
	    //This is done quite unsafe.

	    if (indata.length()>0) {
	    char val1=(char)Integer.parseInt(indata.substring(0,indata.indexOf(",")));
	    char val2=(char)Integer.parseInt(indata.substring(indata.indexOf(",")+1,indata.length()));
	    
	    // This is due to the symmetry of the key- values, this list could therefore be trimmed down.

	    paddingValues[val1]=val2;
	    paddingValues[val2]=val1;

	    }else{
		System.out.println("String is emtpy");
	    }

	}


	}catch(Exception e) {
	    System.out.println("WARNING - - There was an error!");
	    System.out.println("Error: "+e);
	}

	try{

	BufferedReader in = new BufferedReader(new FileReader("replacementCode.txt"));
	String indata=null;

	while(in.ready()) {
	    indata=in.readLine();

	    if (indata.length()>0) {
	    //This is done quite unsafe.
	    char val1=(char)Integer.parseInt(indata.substring(0,indata.indexOf(",")));
	    char val2=(char)Integer.parseInt(indata.substring(indata.indexOf(",")+1,indata.lastIndexOf(",")));
	    char val3=(char)Integer.parseInt(indata.substring(indata.lastIndexOf(",")+1,indata.length()));
	    
	    // This is due to the symmetry of the key- values, this list could therefore be trimmed down.

	    replacementValues[val1][val2]=val3;
	    replacementValues[val2][val1]=val3;

	    replacementValues[val1][val3]=val2;
	    replacementValues[val3][val1]=val2;

	    replacementValues[val2][val3]=val1;
	    replacementValues[val3][val2]=val1;

	    }else{
		System.out.print("::Error - Empty string");
	    }
	}
	}catch(Exception e) {
	    System.out.println("WARNING - - There was an error.!");
	    System.out.println("Error: "+e);
	}
    }


    /***

	This is where the magic happends, it takes a string, applies magic and there is a response to the challenge.

     ***/

    public String calculateValidationKey(String validationString) {
	StringBuffer key=new StringBuffer();

	key.append(validationString.charAt(0));

	//Every char except the first is calculated by the following procedure.
	for (int i=0;i<(validationString.length()-1);i++) {
	    char lastChar=validationString.charAt(i);
	    char lockChar=validationString.charAt(i+1);
	    char coded=replaceValues(lastChar,lockChar);
	    char encrypted=padValue(coded);
	    key.append(encrypted);
	}

	// The first char, depends on the last one.
	char lastChar=key.charAt(key.length()-1);
	char firstChar=key.charAt(0);
	char coded=replaceValues((char)5,firstChar);
	coded=padValue(coded);
	char encrypted=replaceValues(coded,lastChar);
	key.setCharAt(0,encrypted);

	// And we're done!

	return encodeValidationKey(key.toString());
    }

    private char replaceValues (char val1, char val2) {
	if (replacementValues[val1][val2]==0 && val1!=val2) {
	    System.out.println("ERROR: Needed a code-pair which was unknown, authentication will probably not work");
	}
	
	return replacementValues[val1][val2];

    }

    private char padValue(char val1) {
	return paddingValues[val1];
    }


    /***
	   Some characters are reserved, therefore we escape them using a wieard esceaping method.
     ***/

    private String encodeValidationKey(String key) {
	StringBuffer safeKey=new StringBuffer();
	for (int i=0;i<key.length();i++) {
	    char c=key.charAt(i);



	    if (c==124) 
		safeKey.append("/%DCN"+(int)c+"%/");
	    else if (c==36)
		safeKey.append("/%DCN0"+(int)c+"%/");
	    else if (c==126)
		safeKey.append("/%DCN"+(int)c+"%/");
	    else if (c==96)
		safeKey.append("/%DCN0"+(int)c+"%/");
	    else if (c==5)
		safeKey.append("/%DCN00"+(int)c+"%/");	  
	    else if (c==0)
		safeKey.append("/%DCN00"+(int)c+"%/");	  
	    else
		safeKey.append(c);
	}
	return safeKey.toString();
    }

}







