/* DCJavaClient - a DirectConnect client written in Java
 * Copyright (C) 2001 Stefan Grling, stefan@gorling.se
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

import java.io.*;
import java.net.*;
import java.util.*;

import DCConnector;
import DCProtocolHandler;
import GeneralBufferedReader;
import UserInfo;


/***
    @Description This is the runtime-class, starts the threads and listens for commands from the user.
    @Version 0.20
  This is the main class, it creates an instance of the DCConnector and makes a connection.
***/

class DCMain {

    public static void main (String[] argv) throws Exception{


	UserInfo myself=new UserInfo();

	// Create a user record for myself.
	    myself.nick="DCJavaClient";
	    myself.email="";
	   myself.speed="DSL";
	   //	   myself.sharedBytes=Long.parseLong("4503599627370496"); // I used to say that i had 4096 TB, however, some people tend to get mad.
	   myself.sharedBytes=Long.parseLong("4503599627370");
	    myself.description="A beta Java client...";

	try{
	    // The list of hubs is available at:
	    // http://www.neo-modus.com/PublicHubList.config

	    String host="172.16.16.10";
	    int port=411;
	    System.out.println("argv:"+argv.length);
	    if (argv.length==0) { // Present a list of sites if there were no agrumnets.
		URL url=new URL("http://www.neo-modus.com/PublicHubList.config");
							     
		GeneralBufferedReader indata=new GeneralBufferedReader(new InputStreamReader((InputStream)url.getContent()),"|||||");
		try {
		    int i=0;
		    while (i<200) { // only show the first 200.
		    StringTokenizer st=new StringTokenizer(indata.readLine(),"|");

		    System.out.print(st.nextToken()+" address:"+st.nextToken()+" Desc:"+st.nextToken());
		    i++;
		}

		}catch(Exception e) {}

       	    System.out.println("\nSyntax: java DCMain <hostname>");
	    System.exit(0);
	    }

	    if (argv.length>0) 
		host=argv[0];
	    
	    if (argv.length>1) 
		port=Integer.parseInt(argv[1]);

	    DCConnectorThread DCC=new DCConnectorThread(host,port,myself);
	    DCC.start(); // start the thread.

	    while (!DCC.isConnected()) {
		//wait!
	    }

      	    DCProtocolHandler DCP=DCC.getProtocolHandler();

	    String indata=null;
	    BufferedReader stdin=new BufferedReader(new InputStreamReader(System.in));


	    System.out.println("Welcome to DCJavaClient, write 'help' for help");

	    while(true) {
		indata=stdin.readLine();
		try{
		if ("quit".equals(indata)) {
		    System.out.println("Quitting");
		    DCP.logout(); // we don't know that this works...
		    System.exit(0);
		}

		else if (indata.startsWith("search ")) {
		    System.out.println("Making search for:"+indata.substring(7,indata.length()));
		    DCP.makeSearch(indata.substring(7,indata.length()));
		}

		else if (indata.startsWith("search_movie")) {
		    System.out.println("Making search for:"+indata.substring(13,indata.length()));
		    DCP.makeSearch(indata.substring(13,indata.length()),7,1024*1024*20); // search for a movie larger than 20MB
		}

		else if (indata.startsWith("get")) {
		    DownloadManager dm=DCP.getDownloadManager();
		    System.out.println("Getting file:"+indata.substring(4,indata.length()));
		    dm.downloadFile(Integer.parseInt(indata.substring(4,indata.length())));
		}

		else if (indata.startsWith("chat")) {
		    System.out.println("Sending chat message:"+indata.substring(5,indata.length()));
		    DCP.sendChatMessage(indata.substring(5,indata.length()));
		}

		else if (indata.startsWith("private")) {
		    indata=indata.substring(8,indata.length());
		    System.out.println("Sending private message:"+indata);
		    DCP.sendPrivateMessage(indata.substring(indata.indexOf(" ")+1,indata.length()),indata.substring(0,indata.indexOf(" ")));
		}

		else if (indata.startsWith("list results") || indata.startsWith("list r") || indata.startsWith("l r")) {
		    System.out.println("Listing results:");
		    DownloadManager dm=DCP.getDownloadManager();
		    dm.listSearchResults();
		}

		else if (indata.startsWith("list downloads") || indata.startsWith("list d") || indata.startsWith("l d")) {
		    System.out.println("Listing downloads:");
		    DownloadManager dm=DCP.getDownloadManager();
		    dm.listDownloadStatus();
		}

		else if (indata.startsWith("help")) {
		    System.out.println("*****************************************************************************");
		    System.out.println(" Command:           Description:");
		    System.out.println(" search <keyword> - issues a search Request, e.x. search readme.txt");
		    System.out.println(" list results     - lists search results from last search (short list s or l s)");
		    System.out.println(" list downloads   - lists downloads in progress (short list d or l d)");
		    System.out.println(" get <searchID>   - retrieves the file <searchID>, (the id is the number printed first in list results)");
		    System.out.println(" kill <downloadID>   - abort the download <downloadID>, (the id is the number printed first in download status)");
		    System.out.println(" restart <downloadID>   - restarts the download <downloadID>, (the id is the number printed first in download status)");
		    System.out.println(" chat <message>   - sends a message to the general chat window");
		    System.out.println(" private <nick> <message> - sends a private message to <nick>");
		    System.out.println(" quit             - simply leaves");
		    System.out.println("*****************************************************************************");
		}

		else if (indata.startsWith("kill")) {
		    DownloadManager dm=DCP.getDownloadManager();
		    System.out.println("Killing download:"+indata.substring(5,indata.length()));
		    dm.stopDownload(Integer.parseInt(indata.substring(5,indata.length())));
		}

		else if (indata.startsWith("restart")) {
		    DownloadManager dm=DCP.getDownloadManager();
		    System.out.println("Restarting download:"+indata.substring(8,indata.length()));
		    dm.restartDownload(Integer.parseInt(indata.substring(8,indata.length())));
		}

		}catch(Exception e) {}
	    }


	}catch(Exception e) {
	    System.out.println("Caught exception:"+e);
	}

    }    
}


















