/* DCJavaClient - a DirectConnect client written in Java
 * Copyright (C) 2001 Stefan Grling, stefan@gorling.se
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


import DCProtocolHandler;
import java.util.*;
import java.io.*;
import SearchResult;

import DCClientConnectorThread;

/***
    @Description This class keeps track of download threads, search results, pretty much everything related to file download.
    @Version 0.20
 ***/

class DownloadManager {

    DCProtocolHandler dcp=null;
    Vector searchResults=new Vector();
    Vector downloadQueue=new Vector();
    Vector ongoingDownloads=new Vector();
    Vector connectionRequests=new Vector();
    Hashtable blackList=new Hashtable();

    public DownloadManager(DCProtocolHandler dcp) {
	this.dcp=dcp;
    }

    public void clearSearchResults() {
	searchResults.clear();
    }

    public void addSearchResult(String sr) throws IOException{
	UserHandler uh=dcp.getUserHandler();
	SearchResult sresult=new SearchResult(sr);
	System.out.println(searchResults.size()+": "+sresult.filename);
	searchResults.addElement(sresult);
    }

    public void listSearchResults() {
	System.out.println("There was "+searchResults.size()+" items in the queue");
	for (int i=0;i<searchResults.size();i++) {
	    SearchResult sr=(SearchResult)searchResults.elementAt(i);
	    System.out.println(i+":"+sr.filename+" size:"+sr.fileSize);
	}
    }
    
    public void downloadFile(int resultID) {
        System.out.println("Adding "+resultID+" to the download queue");
	downloadQueue.addElement((SearchResult)searchResults.elementAt(resultID));
    }

    /***
	Enqueue the IP for reverse-Name lookup.
     ***/
    public void enqueueConnectionRequest(String host) {
	UserHandler uh=dcp.getUserHandler();

	if (!uh.knowsIP(host) && blackList.get(host)==null) {
	    connectionRequests.addElement(host);
	}
    }

    /***
	The only known way for us to get the IP of a user is to ask for a $RevConnectToMe  which returns a 
	$ConnectToMe request with the IP. After this we have to connect to the IP to se wich user it was sending the request.

	This method is quick and dirty. But the principe is, only check a certain ip one time, only request ip from a nick 3 times.
     ***/



    public void findIPs () throws IOException{
	if (connectionRequests.size()>0) {
	System.out.println("Starting to find "+connectionRequests.size()+"st IP:s");
	UserHandler uh=dcp.getUserHandler();
	//while (connectionRequests.size()>0) {
	    String host=(String)connectionRequests.elementAt(0);
	    connectionRequests.removeElementAt(0);
	    if (host!=null) {
	    if (!uh.knowsIP(host) && blackList.get(host)==null) {
	    DCClientConnector DCCC=new DCClientConnector(host,dcp.myself);
	    DCCC.openConnection();
	    DCClientProtocolHandler DCCP=null;
	    while(DCCP==null) { // waqit for it to ble initialized.
		DCCP=DCCC.getProtocolHandler();
	    }
	    try{
       	    uh.setIP(DCCP.getRemoteNick(),host);	    
	    }catch(IOException e) {
		System.out.println("Addning "+host+" to blacklist");
		blackList.put(host,host);
	    }
	    }
	}
	}
	
    }

    /***
	The actual method which starts downloads. It first tries to associate the SearchResponse with an UserInfo-record and thereafter an UserInfo-record with a IP. When this is done, we simply pass it of to a ClientConnectorThread, stores it in a vector and forgets about it.
     ***/

    public void downloadFiles() throws IOException {
	if (downloadQueue.size()>0) {
	    UserHandler uh=dcp.getUserHandler();
	    for (int i=0;i<downloadQueue.size();i++) {
		SearchResult sr=(SearchResult)downloadQueue.elementAt(i);

		if (sr!=null) {
		if (sr.userInfo==null) {
		    sr.userInfo=uh.getUserInfo(sr.nick);
		}

		if (sr.userInfo==null) {
		    dcp.getInfo(sr.nick);
		}else if(sr.userInfo.ip==null) {
		    if (sr.ipRequests<3) {
		    dcp.requestConnection(sr.userInfo.nick);
		    sr.ipRequests++;
		    }
		}else {
		    DCClientConnectorThread downloader=new DCClientConnectorThread(sr,dcp.myself);
		    downloader.start();
		    ongoingDownloads.addElement(downloader);
		    downloadQueue.set(i,null); // if we remove the element from the same list we're iterating over, we'll get problems.
		}
		}

	    }
	    
	}

    }


    /***
	Loops through the vector of download threads and prints status.
     ***/
    public void listDownloadStatus() {
	System.out.println("Starts to print download status:"+ongoingDownloads.size());
	for (int i=0;i<ongoingDownloads.size();i++) {
	    DCClientConnectorThread dcct=(DCClientConnectorThread)ongoingDownloads.elementAt(i);
	    SearchResult status=dcct.getDownloadStatus();
	    System.out.println(i+":"+status.filename+" "+status.bytesRecieved+"/"+status.fileSize+" inProgress:"+status.transferInProgress+" Finnished:"+status.transferFinnished);
	}
	System.out.println("Done printint donwload status");
    }


    public void restartDownload(int downloadID) {
	DCClientConnectorThread dcct=(DCClientConnectorThread)ongoingDownloads.elementAt(downloadID);
	if (dcct!=null)
	    dcct.restartConnection();
    }

    public void stopDownload(int downloadID) {
	DCClientConnectorThread dcct=(DCClientConnectorThread)ongoingDownloads.elementAt(downloadID);
	if (dcct!=null)
	    dcct.stopConnection();
	ongoingDownloads.removeElementAt(downloadID);
    }

    public void processCommands() throws IOException {
	// This command is repeatedly called by the thread.	
	try {
	findIPs();
	}catch(Exception e) {
	    System.out.println("Error:"+e);
	}
	try {
	downloadFiles();
	}catch(Exception e) {
	    System.out.println("Error:"+e);
	}
    }

}




