/* DCJavaClient - a DirectConnect client written in Java
 * Copyright (C) 2001 Stefan Grling, stefan@gorling.se
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

import java.util.*;
import java.io.*;
import UserInfo;
import DCProtocolHandler;

/***

    @Description This class keeps tracks of the users at the hub.
    @Version 0.20

 ***/


class UserHandler {

    DCProtocolHandler dcp=null;
    Vector userList=new Vector(); // The known users in this hub.
    
    public UserHandler(DCProtocolHandler dcp) {
	this.dcp=dcp;
    }
    
    public void clearUserList() {
	userList.clear();
    }

    /***
	Return a UserInfo record for the user with nick <nick>

     ***/

    public UserInfo getUserInfo(String nick) {
	for (int i=0;i<userList.size();i++) {
	    if (((UserInfo)userList.elementAt(i)).hasNick(nick))
		return (UserInfo)userList.elementAt(i);
	}
	return null;
    }

    public void addUser(UserInfo uinfo) throws IOException{
	userList.add(uinfo);
    }


    /***
	Have we previously processed this IP?
     ***/

    public boolean knowsIP(String IP) {
	for (int i=0;i<userList.size();i++) {
	    if (((UserInfo)userList.elementAt(i)).hasIP(IP)) 
		return true;
	}	    
	return false;
    }

    /***
	Returns true if we already know the IP of that user, otherwise false
     ***/

    public boolean knowsIPForNick(String nick) {
	for (int i=0;i<userList.size();i++) {
	    if (((UserInfo)userList.elementAt(i)).ip!=null && ((UserInfo)userList.elementAt(i)).hasNick(nick)) 
		return true;
	}	    
	return false;
    }


    /*** 
	 Updates ther userList so that the nick, nick gets the IP ip.
     ***/
    public void setIP(String nick,String ip) {
	System.out.println("Setting IP:"+ip+"Nick:"+nick);
	for (int i=0;i<userList.size();i++) {
	    if (((UserInfo)userList.elementAt(i)).hasNick(nick))
		((UserInfo)userList.elementAt(i)).setIP(ip);
      	}
    }


    /***
	Takes the NickList from the server and asks for more info about the user.
     ***/

    public void addUsersFromNickList(String nickList) throws IOException {
	StringTokenizer st=new StringTokenizer(nickList,"$$");

	while (st.hasMoreTokens()) {
	    UserInfo newUser=new UserInfo();
	    String ni=st.nextToken();
	    // we don't actually adds the users here.... simply request more info.
	    dcp.getInfo(ni); // request more info about the user.
	}
    }


    /***
	Add an user into our userList, takes a $MyINFO command data string
     ***/

    public void addUser(String myInfo) throws IOException {
	UserInfo newUser=new UserInfo(myInfo);
	boolean isInList=false;

	for (int i=0;i<userList.size();i++) {
	    if (((UserInfo)userList.elementAt(i)).equals(newUser)) {
		userList.set(i,newUser);
		isInList=true; // maybe we want to replace it?
	    }
	}

	if (!isInList) 
	    addUser(newUser);      
    }


    /*** 
	 Remove an user from our userList 

    **/

    public void removeUser(String nick) {
	for (int i=0;i<userList.size();i++) {
	    UserInfo user=(UserInfo)userList.elementAt(i);
	    if (user.hasNick(nick)) 
		userList.removeElementAt(i);
	}
    }

    
}




